@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/log.c,v 1.9 2006/02/13 11:28:56 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing log files. The code for maintaining datestamped
@@ -55,7 +57,9 @@ static uschar *file_path = US"";
 
 /* The given string is split into sections according to length, or at embedded
 newlines, and syslogged as a numbered sequence if it is overlong or if there is
-more than one line.
+more than one line. However, if we are running in the test harness, do not do
+anything. (The test harness doesn't use syslog - for obvious reasons - but we
+can get here if there is a failure to open the panic log.)
 
 Arguments:
   priority       syslog priority
@@ -70,6 +74,8 @@ write_syslog(int priority, uschar *s)
 int len, pass;
 int linecount = 0;
 
+if (running_in_test_harness) return;
+
 if (!syslog_timestamp) s += log_timezone? 26 : 20;
 
 len = Ustrlen(s);
@@ -312,15 +318,16 @@ open, arrange for automatic closure on exec(), and then return. */
 
 if (*fd >= 0)
   {
-  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
+  (void)fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
   return;
   }
 
 /* Open was not successful: try creating the file. If this is a root process,
 we must do the creating in a subprocess set to exim:exim in order to ensure
 that the file is created with the right ownership. Otherwise, there can be a
-race if an exim process is trying to write to the log at the same time. The use
-of SIGUSR1 by the exiwhat utility can provoke a lot of simultaneous writing. */
+race if another Exim process is trying to write to the log at the same time.
+The use of SIGUSR1 by the exiwhat utility can provoke a lot of simultaneous
+writing. */
 
 euid = geteuid();
 
@@ -348,17 +355,23 @@ else if (euid == root_uid)
     _exit((create_log(buffer) < 0)? 1 : 0);
     }
 
-  /* Wait for the subprocess. If it succeeded retry the open. */
+  /* If we created a subprocess, wait for it. If it succeeded retry the open. */
 
-  while (waitpid(pid, &status, 0) != pid);
-  if (status == 0) *fd = Uopen(buffer, O_APPEND|O_WRONLY, LOG_MODE);
+  if (pid > 0)
+    {
+    while (waitpid(pid, &status, 0) != pid);
+    if (status == 0) *fd = Uopen(buffer, O_APPEND|O_WRONLY, LOG_MODE);
+    }
+
+  /* If we failed to create a subprocess, we are in a bad way. We fall through
+  with *fd still < 0, and errno set, letting the code below handle the error. */
   }
 
 /* If we now have an open file, set the close-on-exec flag and return. */
 
 if (*fd >= 0)
   {
-  fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
+  (void)fcntl(*fd, F_SETFD, fcntl(*fd, F_GETFD) | FD_CLOEXEC);
   return;
   }
 
@@ -732,7 +745,7 @@ va_end(ap);
 this way because it kind of fits with LOG_RECIPIENTS. */
 
 if ((flags & LOG_SENDER) != 0 &&
-    ptr < log_buffer + LOG_BUFFER_SIZE - 8 - Ustrlen(raw_sender))
+    ptr < log_buffer + LOG_BUFFER_SIZE - 10 - Ustrlen(raw_sender))
   {
   sprintf(CS ptr, " from <%s>", raw_sender);
   while (*ptr) ptr++;
@@ -805,7 +818,7 @@ if ((flags & LOG_MAIN) != 0 &&
       uschar *nowstamp = tod_stamp(tod_log_datestamp);
       if (Ustrncmp (mainlog_datestamp, nowstamp, Ustrlen(nowstamp)) != 0)
         {
-        close(mainlogfd);             /* Close the file */
+        (void)close(mainlogfd);       /* Close the file */
         mainlogfd = -1;               /* Clear the file descriptor */
         mainlog_inode = 0;            /* Unset the inode */
         mainlog_datestamp = NULL;     /* Clear the datestamp */
@@ -821,7 +834,7 @@ if ((flags & LOG_MAIN) != 0 &&
       {
       if (Ustat(mainlog_name, &statbuf) < 0 || statbuf.st_ino != mainlog_inode)
         {
-        close(mainlogfd);
+        (void)close(mainlogfd);
         mainlogfd = -1;
         mainlog_inode = 0;
         }
@@ -927,7 +940,7 @@ if (write_rejectlog && (flags & LOG_REJECT) != 0)
       uschar *nowstamp = tod_stamp(tod_log_datestamp);
       if (Ustrncmp (rejectlog_datestamp, nowstamp, Ustrlen(nowstamp)) != 0)
         {
-        close(rejectlogfd);             /* Close the file */
+        (void)close(rejectlogfd);       /* Close the file */
         rejectlogfd = -1;               /* Clear the file descriptor */
         rejectlog_inode = 0;            /* Unset the inode */
         rejectlog_datestamp = NULL;     /* Clear the datestamp */
@@ -944,7 +957,7 @@ if (write_rejectlog && (flags & LOG_REJECT) != 0)
       if (Ustat(rejectlog_name, &statbuf) < 0 ||
            statbuf.st_ino != rejectlog_inode)
         {
-        close(rejectlogfd);
+        (void)close(rejectlogfd);
         rejectlogfd = -1;
         rejectlog_inode = 0;
         }
@@ -988,11 +1001,11 @@ if ((flags & LOG_PROCESS) != 0)
 /* Handle the panic log, which is not kept open like the others. If it fails to
 open, there will be a recursive call to log_write(). We detect this above and
 attempt to write to the system log as a last-ditch try at telling somebody. In
-all cases, try to write to log_stderr. */
+all cases except mua_wrapper, try to write to log_stderr. */
 
 if ((flags & LOG_PANIC) != 0)
   {
-  if (log_stderr != NULL && log_stderr != debug_file)
+  if (log_stderr != NULL && log_stderr != debug_file && !mua_wrapper)
     fprintf(log_stderr, "%s", CS log_buffer);
 
   if ((logging_mode & LOG_MODE_SYSLOG) != 0)
@@ -1022,7 +1035,7 @@ if ((flags & LOG_PANIC) != 0)
       flags |= LOG_PANIC_DIE;
       }
 
-    close(paniclogfd);
+    (void)close(paniclogfd);
     }
 
   /* Give up if the DIE flag is set */
@@ -1042,9 +1055,9 @@ void
 log_close_all(void)
 {
 if (mainlogfd >= 0)
-  { close(mainlogfd); mainlogfd = -1; }
+  { (void)close(mainlogfd); mainlogfd = -1; }
 if (rejectlogfd >= 0)
-  { close(rejectlogfd); rejectlogfd = -1; }
+  { (void)close(rejectlogfd); rejectlogfd = -1; }
 closelog();
 syslog_open = FALSE;
 }
