@@ -1,10 +1,10 @@
-/* $Cambridge: exim/src/src/lookups/pgsql.c,v 1.11 2009/11/16 19:50:38 nm4 Exp $ */
+/* $Cambridge: exim/exim-src/src/lookups/pgsql.c,v 1.4 2006/02/07 11:19:01 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Petr Cech for contributing the original code for these
@@ -13,6 +13,21 @@ socket extension. */
 
 #include "../exim.h"
 #include "lf_functions.h"
+#include "pgsql.h"       /* The local header */
+
+/* We can't just compile this code and allow the library mechanism to omit the
+functions if they are not wanted, because we need to have the PGSQL header
+available for compiling. Therefore, compile these functions only if
+LOOKUP_PGSQL is defined. However, some compilers don't like compiling empty
+modules, so keep them happy with a dummy when skipping the rest. Make it
+reference itself to stop picky compilers complaining that it is unused, and put
+in a dummy argument to stop even pickier compilers complaining about infinite
+loops. */
+
+#ifndef LOOKUP_PGSQL
+static void dummy(int x) { dummy(x-1); }
+#else
+
 
 #include <libpq-fe.h>       /* The system header */
 
@@ -34,7 +49,7 @@ static pgsql_connection *pgsql_connections = NULL;
 
 /* See local README for interface description. */
 
-static void *
+void *
 pgsql_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);    /* Just return something non-null */
@@ -48,7 +63,7 @@ return (void *)(1);    /* Just return something non-null */
 
 /* See local README for interface description. */
 
-static void
+void
 pgsql_tidy(void)
 {
 pgsql_connection *cn;
@@ -61,28 +76,6 @@ while ((cn = pgsql_connections) != NULL)
 }
 
 
-/*************************************************
-*       Notice processor function for pgsql      *
-*************************************************/
-
-/* This function is passed to pgsql below, and called for any PostgreSQL
-"notices". By default they are written to stderr, which is undesirable.
-
-Arguments:
-  arg        an opaque user cookie (not used)
-  message    the notice
-
-Returns:     nothing
-*/
-
-static void
-notice_processor(void *arg, const char *message)
-{
-arg = arg;   /* Keep compiler happy */
-DEBUG(D_lookup) debug_printf("PGSQL: %s\n", message);
-}
-
-
 
 /*************************************************
 *        Internal search function                *
@@ -113,7 +106,7 @@ Arguments:
   server       the server string; this is in dynamic memory and can be updated
   resultptr    where to store the result
   errmsg       where to point an error message
-  defer_break  set TRUE if no more servers are to be tried after DEFER
+  defer_break  TRUE if no more servers are to be tried after DEFER
   do_cache     set FALSE if data is changed
 
 Returns:       OK, FAIL, or DEFER
@@ -250,21 +243,10 @@ if (cn == NULL)
     *errmsg = string_sprintf("PGSQL connection failed: %s",
       PQerrorMessage(pg_conn));
     PQfinish(pg_conn);
+    *defer_break = FALSE;
     goto PGSQL_EXIT;
     }
 
-  /* Set the client encoding to SQL_ASCII, which means that the server will
-  not try to interpret the query as being in any fancy encoding such as UTF-8
-  or other multibyte code that might cause problems with escaping. */
-
-  PQsetClientEncoding(pg_conn, "SQL_ASCII");
-
-  /* Set the notice processor to prevent notices from being written to stderr
-  (which is what the default does). Our function (above) just produces debug
-  output. */
-
-  PQsetNoticeProcessor(pg_conn, notice_processor, NULL);
-
   /* Add the connection to the cache */
 
   cn = store_get(sizeof(pgsql_connection));
@@ -312,6 +294,7 @@ else
     *errmsg = string_sprintf("PGSQL: query failed: %s (%s) (%s)\n",
                              PQresultErrorMessage(pg_result),
                              PQresStatus(PQresultStatus(pg_result)), query);
+    *defer_break = FALSE;
     goto PGSQL_EXIT;
     }
 
@@ -395,16 +378,33 @@ else
 *************************************************/
 
 /* See local README for interface description. The handle and filename
-arguments are not used. The code to loop through a list of servers while the
-query is deferred with a retryable error is now in a separate function that is
-shared with other SQL lookups. */
+arguments are not used. Loop through a list of servers while the query is
+deferred with a retryable error. */
 
-static int
+int
 pgsql_find(void *handle, uschar *filename, uschar *query, int length,
   uschar **result, uschar **errmsg, BOOL *do_cache)
 {
-return lf_sqlperform(US"PostgreSQL", US"pgsql_servers", pgsql_servers, query,
-  result, errmsg, do_cache, perform_pgsql_search);
+int sep = 0;
+uschar *server;
+uschar *list = pgsql_servers;
+uschar buffer[512];
+
+DEBUG(D_lookup) debug_printf("PGSQL query: %s\n", query);
+
+while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
+        != NULL)
+  {
+  BOOL defer_break;
+  int rc = perform_pgsql_search(query, server, result, errmsg, &defer_break,
+    do_cache);
+  if (rc != DEFER || defer_break) return rc;
+  }
+
+if (pgsql_servers == NULL)
+  *errmsg = US"no PGSQL servers defined (pgsql_servers option)";
+
+return DEFER;
 }
 
 
@@ -422,16 +422,6 @@ to allow escaping "on spec". If you use something like "where id="ab\%cd" it
 does treat the string as "ab%cd". So we can safely quote percent and
 underscore. [This is different to MySQL, where you can't do this.]
 
-The original code quoted single quotes as \' which is documented as valid in
-the O'Reilly book "Practical PostgreSQL" (first edition) as an alternative to
-the SQL standard '' way of representing a single quote as data. However, in
-June 2006 there was some security issue with using \' and so this has been
-changed.
-
-[Note: There is a function called PQescapeStringConn() that quotes strings.
-This cannot be used because it needs a PGconn argument (the connection handle).
-Why, I don't know. Seems odd for just string escaping...]
-
 Arguments:
   s          the string to be quoted
   opt        additional option text or NULL if none
@@ -439,7 +429,7 @@ Arguments:
 Returns:     the processed string or NULL for a bad option
 */
 
-static uschar *
+uschar *
 pgsql_quote(uschar *s, uschar *opt)
 {
 register int c;
@@ -457,12 +447,7 @@ t = quoted = store_get(Ustrlen(s) + count + 1);
 
 while ((c = *s++) != 0)
   {
-  if (c == '\'')
-    {
-    *t++ = '\'';
-    *t++ = '\'';
-    }
-  else if (Ustrchr("\n\t\r\b\"\\%_", c) != NULL)
+  if (Ustrchr("\n\t\r\b\'\"\\%_", c) != NULL)
     {
     *t++ = '\\';
     switch(c)
@@ -486,47 +471,6 @@ while ((c = *s++) != 0)
 return quoted;
 }
 
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-pgsql_version_report(FILE *f)
-{
-#ifdef DYNLOOKUP
-fprintf(f, "Library version: PostgreSQL: Exim version %s\n", EXIM_VERSION_STR);
-#endif
-
-/* Version reporting: there appears to be no available information about
-the client library in libpq-fe.h; once you have a connection object, you
-can access the server version and the chosen protocol version, but those
-aren't really what we want.  It might make sense to debug_printf those
-when the connection is established though? */
-}
-
-
-static lookup_info _lookup_info = {
-  US"pgsql",                     /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  pgsql_open,                    /* open function */
-  NULL,                          /* no check function */
-  pgsql_find,                    /* find function */
-  NULL,                          /* no close function */
-  pgsql_tidy,                    /* tidy function */
-  pgsql_quote,                   /* quoting function */
-  pgsql_version_report           /* version reporting */
-};
-
-#ifdef DYNLOOKUP
-#define pgsql_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &_lookup_info };
-lookup_module_info pgsql_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
+#endif  /* PGSQL_LOOKUP */
 
 /* End of lookups/pgsql.c */
