@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/macros.h,v 1.20 2005/11/14 16:09:54 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/macros.h,v 1.24 2006/03/09 15:10:16 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -160,11 +160,6 @@ record. */
 
 #define WAIT_NAME_MAX 50
 
-/* The numbers of connection and message variables for ACLs */
-
-#define ACL_C_MAX 10
-#define ACL_M_MAX 10
-
 /* Fixed option values for all PCRE functions */
 
 #define PCRE_COPT 0   /* compile */
@@ -386,12 +381,13 @@ set all the bits in a multi-word selector. */
 #define LX_rejected_header             0x80001000
 #define LX_return_path_on_delivery     0x80002000
 #define LX_sender_on_delivery          0x80004000
-#define LX_smtp_confirmation           0x80008000
-#define LX_subject                     0x80010000
-#define LX_tls_certificate_verified    0x80020000
-#define LX_tls_cipher                  0x80040000
-#define LX_tls_peerdn                  0x80080000
-#define LX_unknown_in_list             0x80100000
+#define LX_sender_verify_fail          0x80008000
+#define LX_smtp_confirmation           0x80010000
+#define LX_subject                     0x80020000
+#define LX_tls_certificate_verified    0x80040000
+#define LX_tls_cipher                  0x80080000
+#define LX_tls_peerdn                  0x80100000
+#define LX_unknown_in_list             0x80200000
 
 #define L_default     (L_connection_reject        | \
                        L_delay_delivery           | \
@@ -406,6 +402,7 @@ set all the bits in a multi-word selector. */
 
 #define LX_default   ((LX_acl_warn_skipped        | \
                        LX_rejected_header         | \
+                       LX_sender_verify_fail      | \
                        LX_tls_cipher) & 0x7fffffff)
 
 /* Private error numbers for delivery failures, set negative so as not
@@ -455,6 +452,8 @@ to conflict with system errno values. */
 #define ERRNO_AUTHFAIL       (-42)   /* When required by client */
 #define ERRNO_CONNECTTIMEOUT (-43)   /* Used internally in smtp transport */
 #define ERRNO_RCPT4XX        (-44)   /* RCPT gave 4xx error */
+#define ERRNO_MAIL4XX        (-45)   /* MAIL gave 4xx error */
+#define ERRNO_DATA4XX        (-46)   /* DATA gave 4xx error */
 
 /* These must be last, so all retry deferments can easily be identified */
 
